import axios from 'axios';
import { extend, isString } from '@geotify/shared';

class AjaxAxios {
    static _defaultAxiosInstance = axios.create();
    static setDefaultAxiosInstance(instance) {
        this._defaultAxiosInstance = instance;
    }
    _axiosInstance;
    _url;
    _data;
    _params;
    _header = {};
    _config = {};
    constructor(axiosInstance) {
        this._axiosInstance = axiosInstance ?? AjaxAxios._defaultAxiosInstance;
    }
    async mountGet() {
        const config = extend({}, this._config, {
            headers: this._header,
            params: this._params,
            data: this._data,
        });
        const res = await this._axiosInstance.get(this._url, config);
        return res.data;
    }
    async mountPost() {
        const config = extend({}, this._config, {
            headers: this._header,
            params: this._params,
        });
        const res = await this._axiosInstance.post(this._url, this._data, config);
        return res.data;
    }
    url(url) {
        this._url = url;
        return this;
    }
    data(data) {
        this._data = data;
        return this;
    }
    params(params) {
        this._params = params;
        return this;
    }
    header(header) {
        this._header = header;
        return this;
    }
    config(config) {
        this._config = config;
        return this;
    }
}
function createAxios(instance) {
    return new AjaxAxios(instance);
}

class AjaxFetch {
    _url;
    _body;
    _headers = {};
    _config = {};
    _controller = new AbortController();
    url(u) {
        this._url = u;
        return this;
    }
    body(b, type = 'json') {
        if (type === 'json') {
            this.header('Content-Type', 'application/json;charset=utf-8');
            this._body = JSON.stringify(b);
        }
        else if (type === 'form') {
            const formData = new FormData();
            Object.entries(b).forEach(([k, v]) => formData.append(k, v));
            this._body = formData;
        }
        else {
            this._body = b;
        }
        return this;
    }
    headers(h) {
        Object.entries(h).forEach(([k, v]) => this.header(k, v));
        return this;
    }
    header(k, v) {
        this._headers[k] = v;
        return this;
    }
    config(arg0, arg1) {
        if (isString(arg0)) {
            this._config[arg0] = arg1;
        }
        else {
            Object.entries(arg0).forEach(([k, v]) => this._config[k] = v);
        }
        return this;
    }
    execute(config) {
        return fetch(this._url, {
            body: this._body,
            headers: this._headers,
            signal: this._controller.signal,
            ...this._config,
            ...config,
        });
    }
    async methodGet(responseType = 'json') {
        return this.execute({ method: 'get' })
            .then(res => res[responseType]());
    }
    async methodPost(responseType = 'json') {
        return this.execute({ method: 'post' })
            .then(res => res[responseType]());
    }
    cancel() {
        this._controller.abort();
    }
}
function createFetch() {
    return new AjaxFetch();
}

function createFormData() {
    const formData = new FormData();
    const factory = {
        append(...args) {
            formData.append(...args);
            return factory;
        },
        delete(...args) {
            formData.delete(...args);
            return factory;
        },
    };
    return {
        formData,
        factory,
    };
}

function createURLSearchParams() {
    const urlSearchParams = new URLSearchParams();
    const factory = {
        append(...args) {
            urlSearchParams.append(...args);
            return factory;
        },
        delete(...args) {
            urlSearchParams.delete(...args);
            return factory;
        }
    };
    return {
        urlSearchParams,
        factory,
    };
}

export { AjaxAxios, AjaxFetch, createAxios, createFetch, createFormData, createURLSearchParams };
